@@ -5,6 +5,9 @@
  * published by the Free Software Foundation.
  */
 
+/* for AI_IDN */
+#define _GNU_SOURCE
+
 /* System library */
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +32,10 @@
 #include <idna.h>
 #endif
 
+#ifndef AI_IDN
+#define AI_IDN 0
+#endif
+
 /* Application-specific */
 #include "data.h"
 #include "whois.h"
@@ -165,7 +172,6 @@ int main(int argc, char *argv[])
 	free(qstring);
 	qstring = tmp;
 	server = whichwhois(qstring);
-
     }
 
     handle_query(server, port, qstring, fstring);
@@ -180,7 +186,7 @@ int main(int argc, char *argv[])
 const char *handle_query(const char *hserver, const char *hport,
 	const char *qstring, const char *fstring)
 {
-    const char *server, *port = NULL;
+    const char *server = NULL, *port = NULL;
     char *p;
 
     if (hport) {
@@ -236,6 +242,7 @@ const char *handle_query(const char *hserver, const char *hport,
 	    /* XXX should fail if p = 0.0.0.0 */
 	    printf(_("\nQuerying for the IPv4 endpoint %s of a 6to4 IPv6 address.\n\n"), p);
 	    server = whichwhois(p);
+	    /* XXX should fail if server[0] < ' ' */
 	    qstring = p;			/* XXX leak */
 	    break;
 	default:
@@ -453,6 +460,13 @@ char *queryformat(const char *server, const char *flags, const char *query)
 	    && !strchr(query, ' ') && !*flags)
 	sprintf(buf, "-T dn,ace -C US-ASCII %s", query);
     else
+    /* here we have another registrar who could not make things simple
+     * -C sets the language for both input and output
+     */
+    if (!isripe && strcmp(server, "whois.cat") == 0 && domcmp(query, ".cat")
+	    && !strchr(query, ' '))
+	sprintf(buf, "-C US-ASCII ace %s", query);
+    else
 #endif
     if (!isripe && (strcmp(server, "whois.nic.mil") == 0 ||
 	    strcmp(server, "whois.nic.ad.jp") == 0) &&
@@ -491,20 +505,20 @@ int hide_line(int *hiding, const char *const line)
 	}
 	return 0;				/* don't hide this line */
     } else if (*hiding > HIDE_UNSTARTED) {	/* hiding something */
-	if (strncmp(line, hide_strings[*hiding + 1],
-		    strlen(hide_strings[*hiding + 1])) == 0) {
-	    *hiding = HIDE_DISABLED;		/* stop hiding */
-	    return 1;				/* but hide the last line */
+	if (*hide_strings[*hiding + 1] == '\0')	{ /*look for a blank line?*/
+	    if (*line == '\n' || *line == '\r' || *line == '\0') {
+		*hiding = HIDE_DISABLED;	/* stop hiding */
+		return 0;		/* but do not hide the blank line */
+	    }
+	} else {				/*look for a matching string*/
+	    if (strncmp(line, hide_strings[*hiding + 1],
+			strlen(hide_strings[*hiding + 1])) == 0) {
+		*hiding = HIDE_DISABLED;	/* stop hiding */
+		return 1;			/* but hide the last line */
+	    }
 	}
 	return 1;				/* we are hiding, so do it */
-    }
-#if 0
-    /* XXX */
-    /* no match, the action depends on the state stored in *hiding */
-    if (*hiding > HIDE_UNSTARTED)
-	return 1;
-    else
-#endif
+    } else
 	return 0;
 }
 
@@ -519,7 +533,7 @@ const char *do_query(const int sock, const char *query)
     fi = fdopen(sock, "r");
     if (write(sock, query, strlen(query)) < 0)
 	err_sys("write");
-/* Using shutdown breaks the buggy RIPE server.
+/* Using shutdown used to break the buggy RIPE server. Would this work now?
     if (shutdown(sock, 1) < 0)
 	err_sys("shutdown");
 */
@@ -679,9 +693,10 @@ int openconn(const char *server, const char *port)
     memset(&hints, 0, sizeof(struct addrinfo));
     hints.ai_family = AF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_IDN;
 
-    if ((err = getaddrinfo(server, port ? port : "whois", &hints, &res)) != 0)
-	err_quit("getaddrinfo: %s", gai_strerror(err));
+    if ((err = getaddrinfo(server, port ? port : "nicname", &hints, &res)) != 0)
+	err_quit("getaddrinfo(%s): %s", server, gai_strerror(err));
     for (ai = res; ai; ai = ai->ai_next) {
 	if ((fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0)
 	    continue;		/* ignore */
@@ -773,14 +788,39 @@ char *normalize_domain(const char *dom)
 /* server and port have to be freed by the caller */
 void split_server_port(const char *const input,
 	const char **server, const char **port) {
-    char *q, *p;
+    char *p;
 
-    *server = q = strdup(input);
+    if (*input == '[' && (p = strchr(input, ']'))) {	/* IPv6 */
+	char *s;
+	int len = p - input - 1;
 
-    for (p = q; *p && *p != ':'; *q++ = tolower(*p++));
-    if (*p == ':')
-	*port = strdup(p + 1);
-    *p = '\0';
+	*server = s = malloc(len + 1);
+	memcpy(s, input + 1, len);
+	*(s + len) = '\0';
+
+	p = strchr(p, ':');
+	if (p && *(p + 1) != '\0')
+	    *port = strdup(p + 1);			/* IPv6 + port */
+    } else if ((p = strchr(input, ':')) &&		/* IPv6, no port */
+	    strchr(p, ':')) {				/*   and no brackets */
+	*server = strdup(input);
+    } else if ((p = strchr(input, ':'))) {		/* IPv4 + port */
+	char *s;
+	int len = p - input;
+
+	*server = s = malloc(len + 1);
+	memcpy(s, input, len);
+	*(s + len) = '\0';
+
+	if (*(p + 1) != '\0')
+	    *port = strdup(p + 1);
+    } else {						/* IPv4, no port */
+	*server = strdup(input);
+    }
+
+    /* change the server name to lower case */
+    for (p = (char *) *server; *p && *p != '\0'; p++)
+	*p = tolower(*p);
 }
 
 char *convert_6to4(const char *s)
